#ifndef __STACK_HPP_
#define __STACK_HPP_
#ifndef NO_LIBS
#include "jmalloc.hpp"
#else
#include "fakelib.hpp"
#endif

#include <stdio.h>
struct cons_cell;

template<class T> class grow_stack        // stack does not shrink
{ 
  public :
  T **sdata;
  unsigned int son, _max_size;
  /* <sigh> the max_size parameter is the number of bytes of the pointerstack
     instead of the number of entries which it ofcourse should have been.
     This breaks on 64 bit since the caller assumes 4 bytes per pointer and
     thus on 64 bit allocates not enough memory. Instead of fixing all callers
     we work around this by multiplying maxsize by 2 on 64 bit. */
  grow_stack(unsigned int max_size)
  { 
    max_size *= sizeof(void*)/sizeof(int);
    sdata = (T **)jmalloc(max_size, "pointer stack");
    son=0;
    _max_size=max_size;
  }
  
  void push(T *data) 
  {
    sdata[son]=data;
    son++;
    if (son >= (_max_size/sizeof(int)))
    {
      lbreak("stack overflow\n");
      exit(0);
    }
  }
   
  T *pop(unsigned int total) 
  {
    if (total>son) { lbreak("stack underflow\n"); exit(0); }
    son-=total;
    return sdata[son];
  }

  void clean_up() 
  { 
    if (son!=0) fprintf(stderr,"Warning cleaning up stack and not empty\n");
    jfree(sdata); 
    sdata=NULL;  son=0; 
  }
} ;

#endif
